import $ from "jquery";
import HSSelect, { ISelectOptions } from "@preline/select";
import HSDataTable, { IDataTableOptions } from "@preline/datatable";
import DataTable from "datatables.net";
import { ConfigColumnDefs, AjaxSettings } from "datatables.net-dt";
import { autoUpdate, computePosition, offset } from "@floating-ui/dom";
import { formatTimestamp } from "../../../src/ts/main";

// #region DataTable
//

// Fix dependency bugs with preline
(window as any).DataTable = DataTable;
(window as any).$hsDataTableCollection = [];

const emptyTableHtml: string = `
    <div class="max-w-md w-full min-h-[400px] flex flex-col justify-center mx-auto px-6 py-4">
        <div class="flex justify-center items-center size-[46px] bg-gray-100 rounded-lg dark:bg-neutral-800">
            <svg class="shrink-0 size-6 text-gray-600 dark:text-neutral-400" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24" height="24" stroke="currentColor" stroke-width="2" fill="none" stroke-linecap="round" stroke-linejoin="round"><path d="M4 11a9 9 0 0 1 9 9"></path><path d="M4 4a16 16 0 0 1 16 16"></path><circle cx="5" cy="19" r="1"></circle></svg>
        </div>
        <h2 class="mt-5 font-semibold text-gray-800 dark:text-white">
            No results found
        </h2>
        <p class="mt-2 text-sm text-gray-600 dark:text-neutral-400">
            Refine your search or create a new filter.
            Alternatively, use a template to deploy a ready-made filter.
        </p>

        <div class="mt-5 flex flex-col sm:flex-row gap-2">
            <button type="button" class="open-edit-modal-js py-2 px-3 inline-flex justify-center items-center gap-x-2 text-sm font-medium rounded-lg border border-transparent bg-blue-600 text-white hover:bg-blue-700 focus:outline-hidden focus:bg-blue-700 disabled:opacity-50 disabled:pointer-events-none" data-hs-overlay="#TODO">
                <svg class="shrink-0 size-4" xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24" fill="none" stroke="currentColor" stroke-width="2" stroke-linecap="round" stroke-linejoin="round"><path d="M5 12h14"/><path d="M12 5v14"/></svg>
                Create a filter
            </button>
            <button type="button" onclick="alert('not implemented');" class="py-2 px-3 inline-flex justify-center items-center gap-x-2 text-sm font-medium rounded-lg border border-gray-200 bg-white text-gray-800 shadow-xs hover:bg-gray-50 focus:outline-hidden focus:bg-gray-50 disabled:opacity-50 disabled:pointer-events-none dark:bg-neutral-800 dark:border-neutral-700 dark:text-white dark:hover:bg-neutral-700 dark:focus:bg-neutral-700">
                Use a Template
            </button>
        </div>
    </div>
`;

const columnDefs: ConfigColumnDefs[] = [
    // Select checkbox column
    {
        target: 0,
        orderable: false,
        searchable: false,
        render: (_data: unknown, _type: unknown, row: any) => { return `
            <td class="size-px whitespace-nowrap">
                <div class="ps-6 py-4">
                    <label class="rowSelect${row.id}-js" class="flex">
                        <input type="checkbox" id="rowSelect${row.id}-js" class="cj-table-checkbox" data-hs-datatable-row-selecting-individual=""/>
                        <span class="sr-only">Checkbox</span>
                    </label>
                </div>
            </td>
        `}
    },
    {
        target: 1,
        data: "name",
        orderable: true,
        searchable: true,
        render: (data: string) => { return `
            <td class="size-px whitespace-nowrap align-top">
                <span class="cj-table-link max-w-[250px] truncate">
                    ${data}
                </span>
            </td>
        `}
    },
    {
        target: 2,
        data: "value",
        orderable: true,
        searchable: true,
        render: (data: string) => { return `
            <td class="size-px whitespace-nowrap align-top">
                <div class="px-6 py-4 max-w-[600px] truncate">
                    <span class="cj-table-text">
                        ${data}
                    </span>
                </div>
            </td>
        `}
    },
    {
        target: 3,
        data: "matching_algorithm",
        orderable: true,
        searchable: true,
        render: (data: string) => {
            const wrapper = $("<div>").addClass("px-6 py-4");
            const label = $("<span>").addClass("cj-table-text");
            let description: string;

            switch (data) {
                case "ANY":
                    description = "Any Word";
                    break;
                case "ALL":
                    description = "All Words";
                    break;
                case "EXACT":
                    description = "Exact Match";
                    break;
                case "REGEX":
                    description = "Regular Expression";
                    break;
                case "FUZZY":
                    description = "Fuzzy Match";
                    break;
            }

            label.text(description);
            wrapper.append(label);
            return wrapper.get(0);
        }
        // render: (data: string) => { return `
        //     <td class="size-px whitespace-nowrap align-top">
        //         <div class="px-6 py-4">
        //             <span class="cj-table-text">
        //                 ${data}
        //             </span>
        //         </div>
        //     </td>
        // `}
    },
    {
        target: 4,
        data: "is_insensitive",
        orderable: true,
        searchable: true,
        render: data => {
            const wrapper = $("<div>").addClass("px-6 py-4");
            const badge = $("<span>").addClass("py-1 px-1.5 inline-flex items-center text-xs font-medium rounded-full");
            const label = $("<span>");

            if (data) {
                badge.addClass("bg-red-100 text-red-800 dark:bg-red-500/10 dark:text-red-500");
                badge.append(label.text("No"));
            } else {
                badge.addClass("bg-teal-100 text-teal-800 dark:bg-teal-500/10 dark:text-teal-500");
                badge.append(label.text("Yes"));
            }

            wrapper.append(badge);
            return wrapper.get(0);
        }
    },
    {
        target: 5,
        data: "is_whitelist",
        orderable: true,
        searchable: true,
        render: data => {
            const wrapper = $("<div>").addClass("px-6 py-4");
            const badge = $("<span>").addClass("py-1 px-2 inline-flex items-center text-xs font-medium rounded-full");
            const label = $("<span>");

            if (data) {
                badge.addClass("bg-teal-100 text-teal-800 dark:bg-teal-500/10 dark:text-teal-500");
                badge.append(label.text("Whitelist"));
            } else {
                badge.addClass("bg-red-100 text-red-800 dark:bg-red-500/10 dark:text-red-500");
                badge.append(label.text("Blacklist"));
            }

            wrapper.append(badge);
            return wrapper.get(0);
        }
    },
    {
        target: 6,
        data: "created_at",
        orderable: true,
        searchable: false,
        render: (data: string) => { return `
            <td class="size-px whitespace-nowrap align-top">
                <div class="px-6 py-4">
                    <span class="cj-table-text">
                        ${formatTimestamp(data)}
                    </span>
                </div>
            </td>
        `}
    }
];

const ajaxSettings: AjaxSettings = {
    url: `/guild/${1204426362794811453}/filters/api/datatable`,
    type: "POST",
    contentType: "application/json",
    dataSrc: "data",
    data: (data: unknown) => {
        if (data === undefined) return;
        // TODO,
        return JSON.stringify(data);
    }
};

const tableOptions: IDataTableOptions = {
    ajax: ajaxSettings,
    serverSide: true,
    processing: true,
    columnDefs: columnDefs,
    pagingOptions: { pageBtnClasses: "hidden" },
    rowSelectingOptions: { selectAllSelector: "#selectAllBox" },
    language: {
        zeroRecords: emptyTableHtml,
        emptyTable: emptyTableHtml,
        loadingRecords: "Placeholder loading message..."
    },
    rowCallback: (row: HTMLTableRowElement) => {
        $(row).addClass("bg-white dark:bg-neutral-900");
    }
};

const table: HSDataTable = new HSDataTable(
    $("#table").get(0),
    tableOptions
);

// #endregion

// #region Page Size Select
// https://preline.co/plugins/html/advanced-select.html

(window as any).$hsSelectCollection = [];
(window as any)["FloatingUIDOM"] = {
    computePosition: computePosition,
    autoUpdate: autoUpdate,
    offset: offset
};

const pageSelectOptions: ISelectOptions = {
    toggleTag: "<button type=\"button\" aria-expanded=\"false\"></button>",
    optionTemplate: `
        <div class=\"flex justify-between items-center w-full\">
            <span data-title></span>
            <span class=\"hidden hs-selected:block\">
                <svg class=\"shrink-0 size-3.5 text-blue-600 dark:text-blue-500\" xmlns=\"http:.w3.org/2000/svg\" width=\"24\" height=\"24\" viewBox=\"0 0 24 24\" fill=\"none\" stroke=\"currentColor\" stroke-width=\"2\" stroke-linecap=\"round\" stroke-linejoin=\"round\"><polyline points=\"20 6 9 17 4 12\"/></svg>
            </span>
        </div>`,
    toggleClasses: "cj-select-toggle",
    optionClasses: "cj-select-option",
    dropdownClasses: `cj-select-dropdown`,
    dropdownSpace: 10,
    dropdownScope: "parent",
    dropdownPlacement: "top",
    dropdownVerticalFixedPlacement: null
};

const pageSizeSelect: HSSelect = new HSSelect(
    $("#selectPageSize-js").get(0),
    pageSelectOptions
);

// #endregion